@@ -7,9 +7,8 @@ from typing import List, Literal
 
 from aiocomfoconnect import Bridge
 from aiocomfoconnect.const import *
-from aiocomfoconnect.properties import Property
 from aiocomfoconnect.sensors import Sensor
-from aiocomfoconnect.util import bytestring
+from aiocomfoconnect.util import bytearray_to_bits, bytestring
 
 _LOGGER = logging.getLogger(__name__)
 
@@ -17,14 +16,17 @@ _LOGGER = logging.getLogger(__name__)
 class ComfoConnect(Bridge):
     """Abstraction layer over the ComfoConnect LAN C API."""
 
-    INITIAL_SENSOR_DELAY = 1  # 1 second cutoff seems fine
+    INITIAL_SENSOR_DELAY = 2  # 2 seconds cutoff seems fine
 
-    def __init__(self, host: str, uuid: str, callback=None):
+    def __init__(self, host: str, uuid: str, loop=None, sensor_callback=None, alarm_callback=None):
         """Initialize the ComfoConnect class."""
-        super().__init__(host, uuid)
+        super().__init__(host, uuid, loop)
 
-        self.set_callback(self._sensor_callback)  # Set the callback to our _sensor_callback method, so we can proces the callbacks.
-        self._callback = callback
+        self.set_sensor_callback(self._sensor_callback)  # Set the callback to our _sensor_callback method, so we can proces the callbacks.
+        self.set_alarm_callback(self._alarm_callback)  # Set the callback to our _alarm_callback method, so we can proces the callbacks.
+
+        self._sensor_callback_fn = sensor_callback
+        self._alarm_callback_fn = alarm_callback
         self._sensors = {}
         self._sensors_values = {}
         self._sensor_hold = None
@@ -34,19 +36,27 @@ class ComfoConnect(Bridge):
         _LOGGER.debug("Unholding sensors")
         self._sensor_hold = None
 
-        # Emit the current cached values of the sensors, by now, they will have received a second update.
+        # Emit the current cached values of the sensors, by now, they will have received a correct update.
         for sensor_id, sensor in self._sensors.items():
-            self._sensor_callback(sensor_id, self._sensors_values[sensor_id])
+            if self._sensors_values[sensor_id] is not None:
+                self._sensor_callback(sensor_id, self._sensors_values[sensor_id])
 
-    async def connect(self, uuid: str, loop=None):
+    async def connect(self, uuid: str, start_session=True):
         """Connect to the bridge."""
-        loop = loop or asyncio.get_running_loop()
+        await super().connect(uuid)
+
+        self._sensors_values = {}
 
         if self.INITIAL_SENSOR_DELAY:
             _LOGGER.debug("Holding sensors for %s second(s)", self.INITIAL_SENSOR_DELAY)
-            self._sensor_hold = loop.call_later(self.INITIAL_SENSOR_DELAY, self._unhold_sensors)
+            self._sensor_hold = self._loop.call_later(self.INITIAL_SENSOR_DELAY, self._unhold_sensors)
 
-        await super().connect(uuid, loop)
+        if start_session:
+            await self.cmd_start_session(True)
+
+            # Register the sensors again (in case we lost the connection)
+            for sensor in self._sensors.values():
+                await self.cmd_rpdo_request(sensor.id, sensor.type)
 
     async def register_sensor(self, sensor: Sensor):
         """Register a sensor on the bridge."""
@@ -89,7 +99,7 @@ class ComfoConnect(Bridge):
 
     def _sensor_callback(self, sensor_id, sensor_value):
         """Callback function for sensor updates."""
-        if self._callback is None:
+        if self._sensor_callback_fn is None:
             return
 
         sensor = self._sensors.get(sensor_id)
@@ -107,7 +117,21 @@ class ComfoConnect(Bridge):
             val = sensor.value_fn(sensor_value)
         else:
             val = round(sensor_value, 2)
-        self._callback(sensor, val)
+        self._sensor_callback_fn(sensor, val)
+
+    def _alarm_callback(self, node_id, alarm):
+        """Callback function for alarm updates."""
+        if self._alarm_callback_fn is None:
+            return
+
+        if alarm.swProgramVersion <= 3222278144:
+            # Firmware 1.4.0 and below
+            error_messages = ERRORS_140
+        else:
+            error_messages = ERRORS
+
+        errors = {bit: error_messages[bit] for bit in bytearray_to_bits(alarm.errors)}
+        self._alarm_callback_fn(node_id, errors)
 
     async def get_mode(self):
         """Get the current mode."""
@@ -371,10 +395,6 @@ class ComfoConnect(Bridge):
         else:
             raise ValueError("Invalid mode: %s" % mode)
 
-    def get_errors(self):
-        """Get the current errors."""
-        return {nodeId: [ERRORS.get(error) for error in errors] for nodeId, errors in super().get_errors().items()}
-
     async def clear_errors(self):
         """Clear the errors."""
         await self.cmd_rmi_request(bytes([0x82, UNIT_ERROR, 0x01]))
